// ClipboardOwnerDebug.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"


int _tmain(int argc, _TCHAR* argv[])
{
	if (argc!=2 || 0==_wcsicmp(argv[1],L"?") || 0==_wcsicmp(argv[1],L"-h") || 0==_wcsicmp(argv[1],L"/?"))
	{
		printf("ClipboardOwnerDebug by Leo Davidson (www.pretentiousname.com)\n");
		printf("\n");
		printf("Usage:\n");
		printf("  ClipboardOwnerDebug.exe <poll interval in milliseconds>\n");
		printf("\n");
		printf("Example (will poll every 100ms i.e. 10 times a second):\n");
		printf("  ClipboardOwnerDebug.exe 100\n");
		printf("\n");

		return 0;
	}

	int iDelay = _wtoi(argv[1]);
	if (iDelay < 0)
		iDelay = 0;

	std::wstring strExtra;
	std::wstring strProcess;
	std::wstring strClass;
	wchar_t szBuffer[5120];

	const DWORD dwRealTestInterval = 500; // Check we can open the clipboard twice a second.

	DWORD dwStartTicks = ::GetTickCount(); // Assume someone won't leave us running for 50 days. :-)
	DWORD dwLastRealTest = dwStartTicks-(dwRealTestInterval*2);

	bool fOwned = true;
	HWND hWndLastSet = NULL;

	while(true)
	{
		bool fCheckDataOwner = false;
		bool fForcePrint = false;

		if ((::GetTickCount() - dwLastRealTest) > dwRealTestInterval)
		{
			dwLastRealTest = ::GetTickCount();

			::SetLastError(ERROR_SUCCESS);

			if (::OpenClipboard(NULL))
			{
				int iCount = ::CountClipboardFormats();
				::CloseClipboard();
				swprintf(szBuffer, _countof(szBuffer), L" -- %d formats", iCount);
				strExtra = szBuffer;
			}
			else
			{
				DWORD dwErr = ::GetLastError();
				swprintf(szBuffer, _countof(szBuffer), L" -- Error opening the clipboard: %lu", dwErr);
				strExtra = szBuffer;
				fForcePrint = true;
			}

			fCheckDataOwner = true;
		}
		else
		{
			strExtra.clear();
		}

		::SetLastError(ERROR_SUCCESS);

		HWND hWnd = fCheckDataOwner ? GetClipboardOwner() : ::GetOpenClipboardWindow();
		DWORD dwErr = ::GetLastError();

		if (!hWnd && dwErr != ERROR_ACCESS_DENIED)
		{
			if (dwErr != ERROR_SUCCESS)
			{
				wprintf(L"(%10lu ms) Error checking clipboard%s: %lu%s\n", (::GetTickCount()-dwStartTicks), fCheckDataOwner?L" data owner":L"", dwErr, strExtra.c_str());
			}
			else
			{
				if (!fCheckDataOwner)
				{
					if (fForcePrint || fOwned)
					{
						wprintf(L"(%10lu ms) Clipboard not in use.%s\n", (::GetTickCount()-dwStartTicks), strExtra.c_str());
						fOwned = false;
					}
				}
				else
				{
					if (fForcePrint || hWndLastSet != hWnd)
					{
						wprintf(L"(%10lu ms) Clipboard data last set without owner. (Simple data.)%s\n", (::GetTickCount()-dwStartTicks), strExtra.c_str());
					}
				}
			}
		}
		else if (fForcePrint || !fCheckDataOwner || hWndLastSet != hWnd)
		{
			if (!fCheckDataOwner)
				fOwned = true;

			strProcess = L"<n/a>";
			strClass = L"<n/a>";
			szBuffer[0] = L'\0';

			DWORD dwProcess = 0;

			if (!hWnd)
			{
				strProcess = L"<access denied>";
				strClass   = L"<access denied>";
			}
			else
			{
				DWORD dwThread = ::GetWindowThreadProcessId(hWnd, &dwProcess);

				if (dwThread && dwProcess)
				{
					HANDLE hProcess = ::OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, dwProcess);
					if (hProcess)
					{
						if (0 < ::GetProcessImageFileName(hProcess, szBuffer, _countof(szBuffer)))
						{
							const wchar_t *szLastBS = wcsrchr(szBuffer, L'\\');
							const wchar_t *szLastFS = wcsrchr(szBuffer, L'/');
							const wchar_t *szName = szBuffer;
							if (szLastBS && szLastBS > szName) szName = szLastBS+1;
							if (szLastFS && szLastFS > szName) szName = szLastFS+1;
							strProcess = szName;
						}

						::CloseHandle(hProcess);
					}
					else if (::GetLastError() == ERROR_ACCESS_DENIED)
					{
						strProcess = L"<access denied>";
					}
				}

				if (0 < ::GetClassName(hWnd, szBuffer, _countof(szBuffer)))
				{
					strClass = szBuffer;
				}
			}

			wprintf(L"(%10lu ms) Clipboard%s PID: %5lu Process: %-15s Class: %s%s\n", (::GetTickCount()-dwStartTicks), fCheckDataOwner?L" data last set by":L" in use          ", dwProcess, strProcess.c_str(), strClass.c_str(), strExtra.c_str());
		}

		if (!fCheckDataOwner)
			::Sleep(iDelay);
		else
			hWndLastSet = hWnd;
	}

	return 0;
}

